require 'inspect_request/version'
require 'ritm'

# You can create multiple Checker instances (verificator). But it will share the same proxy.
module InspectRequest
  # Checker class
  class Checker
    @started = false
    @session = nil

    class << self
      attr_reader :session
      attr_reader :started
    end

    def initialize
      @results = []
      @verifier_block = proc {}

      self.class.start
      self.class.session.on_request do |req|
        track_result @verifier_block.call req.clone
      end
    end

    # Start the proxy. No need to do this unless you've explicitely shut it down
    def self.start
      return if @started
      @session ||= Ritm::Session.new
      @session.start
      @started = true
      trap('INT') { Checker.shutdown }
      trap('TERM') { Checker.shutdown }
    end

    # Shutdown proxy
    def self.shutdown
      @session.shutdown if @started
      @started = false
    end

    def started?
      self.class.started
    end

    # block should return true/false given an +req+ (webrick request instance)
    # ===== Examples
    #
    #    foo.verify do |req|
    #      req.host == 'example.org'
    #    end
    def verify(&block)
      @verifier_block = block
    end

    # did our verification ever fulfilled?
    def fulfilled?
      res = @results.any? # any true?
      @results.clear
      res
    end

    # Configure proxy. It will also auto-restart the proxy
    # ==== Examples
    #    foo = InspectRequest::Checker.new
    #    foo.configure do
    #      proxy[:bind_port] = 7070
    #    end
    def configure(&block)
      self.class.session.send(:configure, &block)
      self.class.session.shutdown
      self.class.session.start
    end

    private

    def track_result(result)
      @results.push result
    end
  end
end
